	SUBROUTINE TYPE250(TIME,XIN,OUT,T,DTDT,PAR,INFO,ICNTRL,*)

C***************************************************************************
C.	THIS COMPONENT SIMULATES THE THERMAL PERFORMANCE OF A
C.	FLAT-PLATE SOLAR COLLECTOR USING THE MODEL DEVELOPED BY
C.	HOTTEL, WHILLIER, AND BLISS.
C	LAST MODIFIED 3/93 - JWT
C	3/04 - TPM - FOR TRNSYS 16
C	12/05 - DAA - Added units checking with RCHECK
C	02/07 - MRC - Simplified to Type d only
C		- 'flowrate' input changed to 'specific flowrate'
C		- parameter/input defaults corrected and reorganized
C		- variable declarations corrected
C		- Temperature Coefficient is now a -ve value
C***************************************************************************
C.
C.	PARAMETERS
C.	A	COLLECTOR AREA
C.	ALF	ABSORPTANCE OF THE COLLECTOR PLATE SURFACE
C.	BR	TEMPERATURE COEFFICIENT OF THE CELLS
C.	CELLPF	RATIO OF CELL AREA TO ABSORBER AREA
C.	CPF	THERMAL CAPACITANCE OF THE COLLECTOR FLUID
C.	EP	THERMAL EMITTANCE OF THE COLLECTOR PLATE SURFACE 
C.	FP	COLLECTOR GEOMETRY EFFICIENCY FACTOR
C.	NG/XNG	NUMBER OF GLASS COVERS
C.	PVEFF	CELL EFFICIENCY
C.	TR	TEMPERATURE OF CELL REFERENCE EFFICIENCY
C.	UBE	CONTRIBUTION TO UL DUE TO BOTTOM AND EDGE
C.	XKL	PRODUCT OF THE EXTINCTION COEFFICIENT AND THE
C.		THICKNESS OF EACH GLASS COVER
C.
C.	INPUT
C.	ANGLE	TILT OF THE COLLECTOR WITH RESPECT TO HORIZONTAL
C.	FLWRT	COLLECTOR FLUID SPECIFIC FLOWRATE
C.	HBT	INSTANTANEOUS BEAM RADIATION ON THE COLLECTOR SURFACE
C.	HDT	INSTANTANEOUS DIFFUSE RADIATION ON THE COLLECTOR SURFACE
C.	TA	AMBIENT TEMPERATURE
C.	TIN	INLET FLUID TEMPERATURE
C.	THETA1	ANGLE OF INCIDENCE OF RADIATION ON THE COLLECTOR
C.	WIND	WINDSPEED
C.
C.	OUTPUT
C.	TAUALF	PRODUCT OF THE TRANSMITTANCE OF THE GLASS
C.		AND THE ABSORPTANCE OF THE COLLECTOR PLATE SURFACE
C.	TCELL	AVERAGE CELL TEMPERATURE
C.	TOUT	OUTLET FLUID TEMPERATURE
C.	QE	ELECTRICAL POWER OUTPUT
C.	QU	USEFUL ENERGY COLLECTION RATE PER UNIT AREA
C.	UL	OVERALL ENERGY LOSS COEFFICIENT. SEE KLEIN
C.	ULO	APPARENT THERMAL LOSS COEFFICIENT
C.
C.	OTHER
C.	TAU60	TRANSMITTANCE OF THE GLASS COVER SYSTEM AT AN
C.		INCIDENCE ANGLE OF 60 DEGREES, AS ASSUMED FOR DIFFUSE
C.		RADIATION SEE HOTTEL AND WOERTZ
C.	HR	TOTAL RADIATION INCIDENT ON THE TILTED COLLECTOR SURFACE
C.	REFIND	REFRACTIVE INDEX OF THE GLASS
C.	TM	MEAN FLUID TEMPERATURE
C-----------------------------------------------------------------------------------------------------------------------
! Copyright  2004 Solar Energy Laboratory, University of Wisconsin-Madison. All rights reserved.
C-----------------------------------------------------------------------------------------------------------------------
C	USE STATEMENTS
	USE TrnsysFunctions
C-----------------------------------------------------------------------------------------------------------------------
C    REQUIRED BY THE MULTI-DLL VERSION OF TRNSYS
      !DEC$ATTRIBUTES DLLEXPORT :: TYPE250				!SET THE CORRECT TYPE NUMBER HERE

C	TRNSYS DECLARATIONS
	IMPLICIT NONE
	DOUBLE PRECISION XIN,OUT,TIME,PAR,STORED,T,DTDT
	INTEGER*4 INFO(15),NP,NI,NOUT,ND
	INTEGER*4 NPAR,NIN,NDER,IUNIT,ITYPE
	INTEGER*4 ICNTRL, NSTORED
	CHARACTER*3 OCHECK,YCHECK
C-----------------------------------------------------------------------------------------------------------------------
C	USER DECLARATIONS
	PARAMETER (NP=12,NI=8,NOUT=8,ND=0,NSTORED=0)
C-----------------------------------------------------------------------------------------------------------------------
C	REQUIRED TRNSYS DIMENSIONS
	DIMENSION XIN(NI),OUT(NOUT),PAR(NP),YCHECK(NI),OCHECK(NOUT),
     &	STORED(NSTORED),T(ND),DTDT(ND)

C-----------------------------------------------------------------------------------------------------------------------
C	DECLARATIONS AND DEFINITIONS FOR THE USER-VARIABLES
	DOUBLE PRECISION IC,K1,K2,EG,PI,SB,ATR,BTR,CTR,TAU040,REFIND,AR,
     & UBE,A,FP,CPF,ALF,UL,TAU,BR,TR,CELLPF,TAUALF,XNG,EP,ANGLE,XKL,
     & TAU60,FE,UB,CB,UF,UT,TIN,FLWRT,TA,TC,BA,HR,ETAR,ETAA,WIND,HBT,
     & HDT,THETA1,ICT,TILT,V,COSTH1,THETA2,COSTH2,HWIND,TM,TMC,TAC,F,C,
     & STF1,STF2,S,SINC,DENOM,TP,TCELLR,TCELL,TEMP,PR,BETA,PRS,ULAB,ULO,
     & FR,HRT,QU,QE,TOUT,TPR,PRSAVE,BETAS,TIME0,TFINAL,DELT,PVEFF
	INTEGER MODE,NPP,ITER,LUIN,NG
	DIMENSION ATR(3),BTR(3),CTR(3),TAU040(3)
	DATA EG/0.88/,PI/3.1415927/,SB/5.678E-08/
	DATA ATR/-2.9868,-1.4214,-0.74816/
	DATA BTR/-3.7360,-5.7356,-6.5262/
	DATA CTR/4.3541,5.7723,6.3769/
	DATA TAU040/0.92,0.845,0.785/
	DATA IUNIT/0/,REFIND/1.526/,NPP/0/,AR/1./,UBE/0./
C-----------------------------------------------------------------------------------------------------------------------
C	TRNSYS FUNCTIONS
	TIME0=getSimulationStartTime()
	TFINAL=getSimulationStopTime()
	DELT=getSimulationTimeStep()
C-----------------------------------------------------------------------------------------------------------------------
C	SET THE VERSION INFORMATION FOR TRNSYS
	IF(INFO(7).EQ.-2) THEN
		INFO(12)=16
		RETURN 1
	ENDIF
C-----------------------------------------------------------------------------------------------------------------------
C	DO THE VERY LAST CALL OF THE SIMULATION MANIPULATIONS HERE
	IF (INFO(8).EQ.-1) THEN
		RETURN 1
	ENDIF
C-----------------------------------------------------------------------------------------------------------------------
C	PERFORM ANY "AFTER-ITERATION" MANIPULATIONS THAT ARE REQUIRED
	IF(INFO(13).GT.0) THEN
		RETURN 1
	ENDIF
C-----------------------------------------------------------------------------------------------------------------------
C	DO THE VERY FIRST CALL OF THE SIMULATION MANIPULATIONS HERE
	IF (INFO(7).EQ.-1) THEN
C	RETRIEVE THE UNIT AND TYPE NUMBER FOR THIS COMPONENT FROM THE INFO ARRAY
		IUNIT=INFO(1)
		ITYPE=INFO(2)
C	SET SOME INFO ARRAY VARIABLES TO TELL THE TRNSYS ENGINE HOW THIS TYPE IS TO WORK
		INFO(6)=8
		INFO(9)=1
		INFO(10)=0
C	SET THE # OF INPUTS, PARAM AND DERIVATIVES
		MODE=PAR(1)
		MODE=4
		NPAR=INFO(4)
		CALL TYPECK (1,INFO,8,NPAR,0)
		DATA YCHECK/'TE1','MF1','DG1','TE1','IR1','IR1','DG1','VE1'/
		DATA OCHECK/'PW1','PW1','TE1','MF1','DM1','TE1','HT1','HT1'/
		CALL RCHECK(INFO,YCHECK,OCHECK)
		RETURN 1
	ENDIF
C-----------------------------------------------------------------------------------------------------------------------


C	DO THE INITIAL TIMESTEP MANIPULATIONS HERE
	IF (TIME.LT.(TIME0+DELT/2.D0)) THEN
		IUNIT=INFO(1)
		ITYPE=INFO(2)
		ITER=0
		MODE=4
		A=PAR(1)
		FP=PAR(2)
		CPF=PAR(3)
		XNG=PAR(4)
		XKL=PAR(5)
		UBE=PAR(6)
		ALF=PAR(7)
		EP=PAR(8)
		PVEFF=PAR(9)
		BR=PAR(10)
		TR=PAR(11)
		CELLPF=PAR(12)
		UL=0.
		NG=XNG
		TAU60=DEXP(-1.21453*XNG*XKL)*(1.-DEXP((ATR(NG)+(BTR(NG)+
     &	CTR(NG)*0.5)*0.5)*0.5))
		CALL SOLARCELL (TCELLR,TEMP,SINC,PR,BETA,IC,V,ITER,NPP,-1,A,
     &	AR,MODE,LUIN)
		RETURN 1
	ENDIF
C-----------------------------------------------------------------------------------------------------------------------
C	RE-READ THE PARAMETERS IF ANOTHER UNIT OF THIS TYPE HAS BEEN CALLED
	IF(INFO(1).NE.IUNIT) THEN
		IUNIT=INFO(1)
		ITYPE=INFO(2)
		ITER=0
		MODE=4
		A=PAR(1)
		FP=PAR(2)
		CPF=PAR(3)
		XNG=PAR(4)
		XKL=PAR(5)
		UBE=PAR(6)
		ALF=PAR(7)
		EP=PAR(8)
		PVEFF=PAR(9)
		BR=PAR(10)
		TR=PAR(11)
		CELLPF=PAR(12)
		UL=0.
		NG=XNG
		TAU60=DEXP(-1.21453*XNG*XKL)*(1.-DEXP((ATR(NG)+(BTR(NG)+
     &	CTR(NG)*0.5)*0.5)*0.5))
	ENDIF

C-----------------------------------------------------------------------------------------------------------------------
C	RETRIEVE THE CURRENT VALUES OF THE INPUTS
	TIN=XIN(1)
	FLWRT=XIN(2)
	ANGLE=XIN(3)
	TA=XIN(4)
	IF(BR.EQ.0.0)THEN
		BR=0.0000001
	ENDIF	
	TC=TR-1./BR
	IF(TC.EQ.TA) TC=TA+0.1
	BA=1./(TC-TA)
	HBT=XIN(5)
	HDT=XIN(6)
	THETA1=XIN(7)
	WIND=XIN(8)
	ETAR=PVEFF*CELLPF
	ETAA=ETAR*(1.+BR*(TA-TR))
	HR=HBT+HDT
	TAU=0.0001
	IF (THETA1.GT.85.) GO TO 23
	IF (HR.LE.1.0E-10) GO TO 23
	THETA1=THETA1*2.*PI/360.0
	COSTH1=DCOS(THETA1)
	THETA2=DASIN(DSIN(THETA1)/REFIND)
	COSTH2=DCOS(THETA2)
	TAU=TAU040(NG)
	IF (COSTH1.LT.0.766) THEN
		TAU=1.0-DEXP((ATR(NG)+(BTR(NG)+CTR(NG)*COSTH1)*COSTH1)*COSTH1)
	ENDIF
	TAU=HBT/HR*TAU*DEXP(-XNG*XKL/COSTH2)
	TAU=TAU+HDT/HR*TAU60
	IF(XNG.EQ.0)THEN
		TAU=1.0
	ENDIF
23	TAUALF=ALF*TAU
	UL=UL-TAU*HR*ETAA*BA
	ICT=0
	HWIND=5.7+3.8*WIND
	TM=TIN
25	IF (ITER.EQ.0) ICT=ICT+1
	IF (ICT.GT.2) GO TO 45
	TMC=TM+273.15
	TAC=TA+273.15
	IF (TMC.LE.TAC) TMC=TAC+1.0
	F=(1.0-0.04*HWIND+5.0E-04*HWIND*HWIND)*(1.0+0.091*XNG)
	IF (XNG.LT..5) F=1.
	C=365.9*(1.0-0.00883*ANGLE+0.0001298*ANGLE*ANGLE)
	STF1=C/TMC*((TMC-TAC)/(XNG+F))**0.33
	STF1=XNG/STF1+1.0/HWIND
	STF1=1.0/STF1
	STF2=1.0/(EP+0.05*XNG*(1.0-EP))+(2.*XNG+F-1.)/EG-XNG
	STF2=SB*(TMC*TMC+TAC*TAC)*(TMC+TAC)/STF2
	UL=(STF1+STF2)*3.6+UBE
	UL=UL-TAU*HR*ETAA*BA
	IF (FLWRT*A-1.E-5) 34,34,30
30	IF ((FP*UL/(FLWRT*CPF)).LT.0.01) GO TO 31
	FR=FLWRT*CPF*(1.0-DEXP(-FP*UL/(FLWRT*CPF)))/UL
	GO TO 32
31	FR=FP
32	HRT=HR*(1.-ETAA/ALF)
	QU=FR*(HRT*TAUALF-UL*(TIN-TA))
	QE=TAU*HR*ETAA*(1.-BA*(FR*(TIN-TA)+(TAUALF*HRT/UL)*(1.-FR)))
	TOUT=QU/FLWRT/CPF+TIN
	GO TO 36
34	QU=0.0
	ULO=UL+TAU*HR*ETAA*BA
	QE=TAU*HR*ETAA*(ULO-TAUALF*HR*BA)/UL
	FLWRT=0.0
	TOUT=(TAUALF*HR-QE)/ULO+TA
36	TM=(TIN+TOUT)/2.0
	TCELL=TM
	IF(HR.LE.1.E-5)THEN
		TCELL=TA
		GO TO 25
		IF(ETAR.LT.1.E-5)THEN
			TCELL=TC
			GO TO 25
		ENDIF
		IF(TAU.LE.1.E-5)THEN
			TCELL=TA
			GO TO 25
		ENDIF
	ENDIF
	TCELL=TC-(QE/(TAU*HR*ETAR))*(TC-TR)
	GO TO 25
45	CONTINUE
	OUT(1)=QU*A
	OUT(2)=QE*A
	OUT(3)=TOUT
	OUT(4)=FLWRT*A
	OUT(5)=TAUALF
	OUT(6)=TCELL
	OUT(7)=UL
	OUT(8)=ULO
46	CONTINUE
	RETURN 1
	END

C-----------------------------------------------------------------------------------------------------------------------
